# Copyright (C) 2010-2016 Svetlana A. Tkachenko <svetlana@members.fsf.org>
# Copyright (C) 2010-2016 David Vo <david.vo2@gmail.com>
# Copyright (C) 2011 FurryHead <furryhead14@yahoo.com>
#
# This file is part of guppy.
#
# guppy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# guppy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with guppy.  If not, see <http://www.gnu.org/licenses/>.



@plugin
class Printer(object):
    def __init__(self, server):
        self.server = server
        self.commands = []
        self.log = server.log

        self.server.handle("message", self.handle_message)
        self.server.handle("kick", self.handle_kick)
        self.server.handle("part", self.handle_part)
        self.server.handle("join", self.handle_join)
        self.server.handle("quit", self.handle_quit)
        self.server.handle("mode", self.handle_mode)
        self.server.handle("notice", self.handle_notice)
        self.server.handle("connect", self.handle_connect)
        self.server.handle("disconnect", self.handle_disconnect)
        self.server.handle("action", self.handle_action)
        self.server.handle("nick", self.handle_nick)
        self.server.handle("error", self.handle_error)

    def handle_connect(self, server):
        self.log("I have connected to " + server, 'irc')

    def handle_disconnect(self, server):
        self.log("I have disconnected from " + server, 'irc')

    def handle_message(self, channel, user, message):
        self.log("<%s> %s: %s" % (channel, user, message), 'irc')

    def handle_action(self, channel, user, action):
        self.log("<%s> * %s %s" % (channel, user, action), 'irc')

    def handle_join(self, channel, user):
        self.log("%s has joined %s" % (user, channel), 'irc')

    def handle_part(self, channel, user, message):
        self.log("%s has left %s (%s)" % (user, channel, message), 'irc')

    def handle_error(self, message):
        self.log("ERROR: " + message, 'irc')

    def handle_quit(self, user, message):
        self.log("%s has quit (%s)" % (user, message), 'irc')

    def handle_nick(self, oldnick, newnick):
        self.log("%s is now known as %s" % (oldnick, newnick), 'irc')

    def handle_kick(self, channel, user, userkicked, message):
        self.log("%s has kicked %s from %s (%s)" % (user, userkicked, channel, message), 'irc')

    def handle_notice(self, user, dest, message):
        self.log("(%s) -%s- %s" % (dest, user, message), 'irc')

    def handle_mode(self, channel, user, mode, otheruser):
        self.log("%s set mode %s on %s" % (user, mode, otheruser if otheruser != "" else channel), 'irc')
